using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml.Serialization;
using System.Windows.Forms;

namespace TranslaterTool
{

    public struct LanguageInfo
    {
        public String ident;
        public String name;
    }

    public static class Translater
    {
        private static String language;
        private static String langFile;
        private static translation trad;

        public static  List<LanguageInfo> GetAvailableLanguage()
        {
            List<LanguageInfo> list = new List<LanguageInfo>();
            foreach (translationLang lang in trad.languages)
            {
                LanguageInfo info = new LanguageInfo();
                info.ident = lang.id;
                info.name = lang.description;
                list.Add(info);
            }
            return list;
        }

        public static String Language
        {
            get { return language; }
            set { language = value; }
        }

        public static bool LoadTranslationFile(String filename)
        {
            if (File.Exists(filename))
            {
                langFile = filename;
                LoadFile();
                return true;
            }
            else
            {
                return false;
            }
        }

        private static void LoadFile()
        {
            StreamReader reader = new StreamReader(langFile);
            XmlSerializer serializer = new XmlSerializer(typeof(translation));

            trad = (translation)serializer.Deserialize(reader);
            language = trad.@default;
        }

        private static bool TranslateMenu(MenuStrip menu)
        {
            menu.Text = GetTranslation(menu.Name);
            foreach (ToolStripMenuItem item in menu.Items)
            {
                TranslateItem(item);
            }
            return true;
        }

        private static void TranslateItem(ToolStripMenuItem item)
        {
            item.Text = GetTranslation(item.Name);
            foreach (ToolStripMenuItem i in item.DropDownItems)
            {
                TranslateItem(i);
            }
        }

        public static bool TranslateForm(Form f)
        {
            f.Text = GetTranslation(f.Name);

            if (f.HasChildren)
            {
                foreach (Form child in f.MdiChildren)
                {
                    TranslateForm(child);
                }
            }

            foreach (Control c in f.Controls)
            {
                if (c is MenuStrip)
                {
                    TranslateMenu((MenuStrip)c);
                }
                else
                {
                    TranslateControl(c);
                }
            }
            return true;
        }

        private static void TranslateControl(Control c)
        {
            c.Text = GetTranslation(c.Name);
            foreach (Control co in c.Controls)
            {
                TranslateControl(co);
            }
        }

        public static String GetTranslation(String text)
        {
            String translation = text;
            for (int i = 0; i < trad.item.Length; i++)
            {
                if ((trad.item[i].id == text))
                {
                    foreach(translationItemTranslate t in trad.item[i].translate)
                    {
                        if (t.lang == language)
                        {
                            return t.text;
                        }
                    }
                }
            }
            return translation;
        }        
    }
}
